#ifndef THERMLIBH

#define THERMLIBH

#define TRUE	1
#define FALSE	0

#define COM_READ_ROM			0x33
#define COM_MATCH_ROM		0x55
#define COM_SKIP_ROM			0xCC
#define COM_SEARCH_ROM		0xF0
#define COM_ALARM_SEARCH	0xEC

#define COM_WR_SP				0x4E
#define COM_RD_SP				0xBE
#define COM_CP_SP				0x48
#define COM_CNVT_T			0x44
#define COM_RC_EE				0xB8
#define COM_RD_PS				0xB4

#define STAT_CRC_INVALID		0x0001
#define STAT_NO_DEVICE_FOUND	0x0002
#define STAT_THERM_BUSY			0x0004
#define STAT_PARASITE_POWER	0x0008
#define STAT_BUS_SHORT_LOW		0x0010
#define STAT_BUS_SHORT_HIGH	0x0020
#define STAT_ALARM_CONDITION	0x0040

#define PORT_O1_BSY1B	0x80
#define PORT_O2_BSY2B	0x10
#define PORT_O1_OR_O2	0x90
#define PORT_ENIB			0x02
#define PORT_D_CLKB		0x01
#define PORT_RESB			0x02
#define SET_ENIB_HIGH	(dataPortC &= ~PORT_ENIB)
#define SET_ENIB_LOW		(dataPortC |= PORT_ENIB)
#define SET_D_CLKB_HIGH	(dataPortA |= PORT_D_CLKB)
#define SET_D_CLKB_LOW 	(dataPortA &= ~PORT_D_CLKB)
#define SET_RESB_HIGH	(dataPortA |= PORT_RESB)
#define SET_RESB_LOW		(dataPortA &= ~PORT_RESB)

// For DS1480, 1-wire reset is D_CLKB high, RESB low
#define PORT_RESET_DATA		0xFD
// For DS1480, 1-wire write 1, read is D_CLKB high, RESB high
#define PORT_WRITE1_DATA	0xFF
// For DS1480, 1-wire write 0 is D_CLKB low, RESB high
#define PORT_WRITE0_DATA	0xFE

#define CHECKTIME		0x00
#define INIT1WTIME	0x03 // Initialize time out value for 1-wire

#define NAME_LEN	30
#define MAX_PARTS	10
#define MAX_NAMES	20

typedef unsigned char	uchar;
typedef unsigned int		uint;
typedef unsigned long	ulong;

class	ThermLib
{
	public:
		uchar	CRCValue;
		uchar	dataPortA;
		uchar	dataPortC;
		int	numLPT;
		int	portLPT[4];
		uchar	ROMCode[8];
		uchar	SPData[9];

		void	InitThermLib( void );
		int	WrSP( uchar TH, uchar TL );
		int	RdSP( uchar *data );
		int	CnvtT( void );
		int	CpSP( void );
		int	RcEE( void );
		int	RdPS( void );
		int	OneWireReset( void );
		int	OneWireWriteByte( int data );
		int	OneWireWriteBit( int data );
		void	CalcCRC( uchar *CRCVal, uchar value );
		int	ValidCRC( uchar code_len, uchar *code );
		int	ValidSPData( void )
					{ return ValidCRC( 9, SPData ); }
		int	ValidROMCode( void )
					{ return ValidCRC( 8, ROMCode ); }
		void	ROMString( char *code_string );
		void	SetCode( uchar *code );
		void	SetCode( void );
		int	SameCode( uchar *code );
		int	ReadStatus( int mask );
		void	SetStatus( int flags );
		void	ClearStatus( int mask );
		float	ReadTemperature( void );

	protected:
		int	status;
		ulong far *SysClock;
		ulong far InitTime;

		void	FindPort( void );
		int	TimeOut( int TCmd );
		int	ParallelCom( uchar type );
};

class	ThermPart : public ThermLib
{
	public:
		char	ROMName[NAME_LEN];

		ThermPart( int );
		int	MatchROM( void );
		void	WriteSP( uchar TH, uchar TL )
				{ SetStatus( MatchROM() ); WrSP( TH, TL ); }
		void	ReadSP( void )
				{ SetStatus( MatchROM() ); SetStatus( RdSP( SPData ) ); }
		void	ConvertT( void )
				{ SetStatus( MatchROM() ); SetStatus( CnvtT() ); }
		void	CopySP( void )
				{ SetStatus( MatchROM() ); CpSP(); }
		void	RecallEE( void )
				{ SetStatus( MatchROM() ); SetStatus( RcEE() ); }
		void	ReadPS( void )
				{ SetStatus( MatchROM() ); SetStatus( RdPS() ); }
		void	SetName( char *name )
				{ strncpy( ROMName, name, NAME_LEN ); }
		int	EvalAlarmCond( void );
		void	AddAlarm( int cond );

	protected:
		int	AlarmState[3];
		int	AlarmCond;
};

class ROMNameTable
{
	public:
		ROMNameTable( void );
		~ROMNameTable( void );

		void	AddROMName( uchar *code, char *name );
		void	FindROMName( uchar *code, char *name );

		int	num_names;
		uchar	**ROMCodeList;
		char	**ROMNameList;
		int	SameCode( uchar *code1, uchar *code2 );
};

class	ThermBus : public ThermLib
{
	public:
		ThermPart* DS1820[MAX_PARTS];
		int	num_parts;
      ROMNameTable* ROMNames;

		ThermBus( int );
		~ThermBus( void );
		void	SetPort( int );
		void	WriteSP( uchar TH, uchar TL )
				{ SetStatus( SkipROM() ); WrSP( TH, TL ); }
		void	ReadSP( void )
				{ SetStatus( SkipROM() ); SetStatus( RdSP( SPData ) ); }
		void	ConvertT( void )
				{ SetStatus( SkipROM() ); SetStatus( CnvtT() ); }
		void	CopySP( void )
				{ SetStatus( SkipROM() ); CpSP(); }
		void	RecallEE( void )
				{ SetStatus( SkipROM() ); SetStatus( RcEE() ); }
		void	ReadPS( void )
				{ SetStatus( SkipROM() ); SetStatus( RdPS() ); }
		void	ReadROM( void );
		void	SearchROM( void );
		void	AlarmSearch( void );

	protected:
		int	SkipROM( void );
		int	SearchBit( void );
		int	FindROM( int busCom, uchar *CurROM,
					int *lastDiscrep, int *done, int num );
		void	SetROMBit( uchar *CurROM, int bitPos, int value );
};

#endif  // #ifndef THERMLIBH 
